<?php

use ZondacryptoPay\Payment\ZondacryptoPay;
use ZondacryptoPay\Payment\Helper;
use ZondacryptoPay\Payment\Payment;
use ZondacryptoPay\Payment\Util;

/**
 * Class WC_Gateway_ZondacryptoPay
 */
class WC_Gateway_ZondacryptoPay extends WC_Payment_Gateway
{

	/**
	 * @var string
	 */
	const PAYMENT_METHOD = 'zondacryptopay';

	/**
	 * initialise gateway with custom settings
	 */
	public function __construct()
	{

		$this->id = self::PAYMENT_METHOD;
		$this->icon = self::get_image_src('logo.svg');
		$this->has_fields = false;
		$this->description = __('Cryptocurrency payments.', 'zondacryptopay');
		$this->title = $this->get_option('zondacryptopay_payment_title')
			?: 'zondacrypto pay';

        $this->supports = array('products');

		// Admin
		$this->method_title = __('ZondacryptoPay', 'zondacryptopay');
		$this->method_description = __('Cryptocurrency payments.', 'zondacryptopay');

		$this->init_form_fields();
		$this->init_settings();

		//Actions
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, [
			$this,
			'process_admin_options',
		]);
		add_action('woocommerce_receipt_' . $this->id, [
			$this,
			'receipt_page',
		]);
		add_action('woocommerce_api_' . strtolower(get_class($this)), [
			$this,
			'zondacryptopay_notification',
		]);

		//Filters
		add_filter('woocommerce_available_payment_gateways', [
			$this,
			'check_zondacryptopay_available_payment_gateways',
		]);
	}

	/**
	 * @return string
	 */
	private static function get_notification_url()
	{
		return add_query_arg('wc-api', 'wc_gateway_zondacryptopay', home_url('/'));
	}

	/**
	 * @return string
	 */
	private static function get_language()
	{

		return get_locale() === 'pl_PL'
			? 'pl'
			: 'en';
	}

	/**
	 * @param string $file
	 *
	 * @return string
	 */
	public function get_image_src($file)
	{
		return WOOCOMMERCE_ZONDACRYPTOPAY_PLUGIN_URL . 'resources/images/' . $file;
	}

	/**
	 * Initialize form in admin panel.
	 */
	public function init_form_fields()
	{
		$this->form_fields = [
			'zondacryptopay_enabled' => [
				'title'   => __('Enable / Disable', 'zondacryptopay'),
				'type'    => 'checkbox',
				'label'   => __('Enable plugin', 'zondacryptopay'),
				'default' => 'yes',
			],

			'zondacryptopay_private_key'   => [
				'title'   => __('Private key', 'zondacryptopay'),
				'type'    => 'text',
				'default' => '',
			],
			'zondacryptopay_public_key'    => [
				'title'   => __('Public key', 'zondacryptopay'),
				'type'    => 'text',
				'default' => '',
			],
			'zondacryptopay_payment_title' => [
				'title'       => __('Payment title', 'zondacryptopay'),
				'type'        => 'text',
				'placeholder' => $this->title,
			],
			'zondacryptopay_notification'  => [
				'title'       => __('Notification URL', 'zondacryptopay') . ': ',
				'type'        => 'title',
				'description' => self::get_notification_url(),
			],
		];
	}

	/**
	 * @param int $order_id
	 *
	 * @return array
	 */
	public function process_payment($order_id)
	{

		$order = new WC_Order($order_id);

		// Return thank you redirect
		return [
			'result'   => 'success',
			'redirect' => $order->get_checkout_payment_url(true),
		];
	}

	/**
	 * @param string $order_id
	 *
	 * @throws Exception
	 */
	public function receipt_page($order_id)
	{

		header("Cache-Control: no-cache, no-store, must-revalidate");

		if(empty(WC()->cart->cart_contents)) {
			wp_redirect(get_home_url());

			return;
		}

		global $wp_query;

		$payment = new Payment(
			$this->get_option('zondacryptopay_private_key'),
			$this->get_option('zondacryptopay_public_key')
		);

		$order = new WC_Order($order_id);

		try {
			$paymentResult = $payment->buildOrder(
				$order->get_currency(),
				$order_id,
				(float) $order->get_total(),
				$order->get_billing_email(),
				self::get_notification_url(),
				$this->get_return_url($order),
				$this->get_return_url($order),
			);
		} catch(Exception $e) {

			$lang = self::get_language();

			$logger = wc_get_logger();
			$logger->error(Helper::getErrorMessage($e->getMessage(), $lang) . ' ' . current_filter() . PHP_EOL);

			$wp_query->query_vars['error_message'] = Helper::getErrorMessage(Helper::ERROR_MESSAGE_FOR_CUSTOMER, $lang);
			$wp_query->query_vars['error'] = true;

			load_template(__DIR__ . '/templates/redirect.php', false);

			return;
		}

		$wp_query->query_vars['error'] = false;

		$wp_query->query_vars['paymentResult'] = $paymentResult;

		// Remove cart
		WC()->cart->empty_cart();

		// region Send email
		WC()->mailer();
		$mail = new WC_Email_New_Order();
		$mail->trigger($order_id);

		$mail2 = new WC_Email_Customer_Processing_Order();
		$mail2->trigger($order_id);
		// endregion

		load_template(__DIR__ . '/templates/redirect.php', false);
	}

	/**
	 * Method that checking notify and changing status
	 *
	 * @throws Exception
	 */
	public function zondacryptopay_notification()
	{
        $logger = wc_get_logger();

		if(!($payloadDecoded = ZondacryptoPay::validateNotification())) {
            $logger->error('Notification validation error' . PHP_EOL);
            ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

			return null;
		}

		$orderId = $payloadDecoded['orderId'];

		if(!($order = wc_get_order($payloadDecoded['orderId']))) {
            $logger->error(sprintf('Invalid orderId: %s', $orderId) . PHP_EOL);
            ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

			return null;
		}

		$order_status = $order->get_status();

		if(($order->post->post_type !== 'shop_order' && $order->post->post_type !== 'shop_order_placehold' ) || ($order_status !== 'pending' && $order_status !== 'failed')) {
            $logger->error(sprintf('Invalid order status: %s and post_type: %s on orderId: %s', $order_status, $order->post->post_type, $orderId) . PHP_EOL);
            ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

			return null;
		}

		$transactionApi = ZondacryptoPay::checkTransaction(
			$payloadDecoded['paymentId'],
			$this->get_option('zondacryptopay_private_key'),
			$this->get_option('zondacryptopay_public_key')
		);
		
		if(!$transactionApi) {
            $logger->error(sprintf('REST response validation error on orderId: %s', $orderId) . PHP_EOL);
            ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

			return null;
		}

		$paidAmount = $transactionApi['data']['amountInDestinationCurrency'];
		if($paidAmount < (float) $order->get_total()) {
            $logger->error(sprintf('Invalid price on orderId: %s, expected: %s, received: %s', $orderId, $order->get_total(), $paidAmount) . PHP_EOL);
            ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

			return null;
		}

		switch($transactionApi['data']['status']) {
            case Util::TRANSACTION_STATUS_COMPLETED:
				wc_reduce_stock_levels($order->get_id());
				$status = 'processing';
				break;
            case Util::TRANSACTION_STATUS_ACTION_REQUIRED:
				$order->add_order_note(
					\sprintf(
						__('<b>zondacrypto pay</b>: Payer action is required. Need to include information needed for <a target="_blank" href="%1s">Travel Rule</a> message: <a target="_blank" href="%2s">%2s</a>.', 'zondacryptopay'),
						'https://zondacrypto.com/en/blog/travel-rule-in-zondacrypto-pay',
                        $payloadDecoded['travelRuleFormUrl'] ?? ''
					)
				);
				ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_200);
				return null;
			default:
                $logger->error(sprintf('REST response on orderId: %s with unexpected status: %s', $orderId, $transactionApi['data']['status']) . PHP_EOL);
                ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

				return null;
		}

		$order->update_status($status);
        ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_200);
		exit;
	}

	/**
	 * Validate data on checkout page. Remove ZondacryptoPay when there is an error.
	 *
	 * @param array $gateways
	 *
	 * @return array
	 */
	public function check_zondacryptopay_available_payment_gateways($gateways)
	{
		if(!$this->is_available()) {
			unset($gateways["zondacryptopay"]);
		}

		return $gateways;
	}

    public function is_available() {
        return Util::canUseForCurrency(get_woocommerce_currency())
            && !empty($this->get_option('zondacryptopay_public_key'))
            && !empty($this->get_option('zondacryptopay_private_key'));
    }
}
